from window_default import WindowDefaultWidget

from users.menu import UsersMenuWidget

from PyQt5.QtWidgets import QLabel, QPushButton


class UsersMainWidget(WindowDefaultWidget):
    selected_user = {'name': None, 'account_id': None}

    def __init__(self, protocol, parent=None):
        super().__init__(parent)

        self.protocol = protocol
        self.protocol.users.connect(self.userAnswer)

        self.makeInput()
        self.makeKeyboard()
        self.setInfo('')

        self.upcase = True
        self.charsUpdate()

        self.waiting_for_users = False
        self.done.clicked.connect(UsersMenuWidget(self).show)

    def makeInput(self):
        info = QLabel('|', self)
        info.setGeometry(169, 11, 567-169, 50)
        info.setObjectName('info')
        self.info = info

        done = QPushButton(None, self)
        done.setGeometry(570, 11, 638-570, 50)
        done.setObjectName('done')
        done.hide()
        self.done = done

        line1 = QPushButton(None, self)
        line1.setObjectName('line1')
        line1.setGeometry(169, 66, 423-169, 117-66)
        line1.clicked.connect(self.lineClick)
        line1.hide()
        self.line1 = line1

        line2 = QPushButton(None, self)
        line2.setObjectName('line2')
        line2.setGeometry(169, 117, 423-169, 117-66)
        line2.clicked.connect(self.lineClick)
        line2.hide()
        self.line2 = line2

        up = QPushButton(None, self)
        up.setObjectName('up')
        up.setGeometry(423, 66, 489-423, 117-66)
        up.clicked.connect(self.upClick)
        up.hide()
        self.up = up

        down = QPushButton(None, self)
        down.setObjectName('down')
        down.setGeometry(423, 117, 489-423, 117-66)
        down.clicked.connect(self.downClick)
        down.hide()
        self.down = down

    def makeKeyboard(self):
        lines = [
            {'dx': 0,  'keys': _('йцукенгшщзхъ')},
            {'dx': 33, 'keys': _('фывапролджэ')},
            {'dx': 66, 'keys': _('ячсмитьбюё')}
        ]

        y = 176

        self.chars = []

        for line in lines:
            x = line['dx']

            for key in line['keys']:
                button = QPushButton(key, self)
                button.setGeometry(x, y, 67, 80)
                button.clicked.connect(self.charClick)
                self.chars.append(button)
                x += 66

            y += 79

        y = 418

        button = QPushButton(None, self)
        button.setGeometry(33, y, 66, 50)
        button.setObjectName('shift')
        button.clicked.connect(self.shiftClick)
        self.upcase = False

        button = QPushButton(' ', self)
        button.setGeometry(104, y, 563-104, 50)
        button.setObjectName('space')
        button.clicked.connect(self.charClick)

        button = QPushButton('-', self)
        button.setGeometry(568, y, 635-568+1, 50)
        button.clicked.connect(self.charClick)

        button = QPushButton('.', self)
        button.setGeometry(635, y, 700-635+1, 50)
        button.clicked.connect(self.charClick)

        button = QPushButton(None, self)
        button.setGeometry(700, y, 766-700, 50)
        button.setObjectName('delete')
        button.clicked.connect(self.deleteClick)

    def charsUpdate(self):
        for char in self.chars:
            if self.upcase:
                char.setText(char.text().upper())
            else:
                char.setText(char.text().lower())

    def getUsers(self):
        self.users = []
        self.offset = 0
        self.waiting_for_users = True
        self.protocol.getUserName(self.text)
        self.done.hide()
        self.line1.hide()
        self.line2.hide()
        self.up.hide()
        self.down.hide()

    def showLine(self, line, offset):
        user = self.users[self.offset + offset]

        line.setText(user['name'])
        line.setProperty('user', user)

    def showUsers(self):
        lng = len(self.users)

        if lng == 0:
            return

        self.line1.show()
        self.showLine(self.line1, 0)

        if lng > 1:
            self.line2.show()
            self.showLine(self.line2, 1)

        if lng > 2:
            self.up.show()
            self.down.show()

    def setInfo(self, text):
        self.text = text
        self.info.setText('%s|' % text)

    # Button handlers #

    def charClick(self):
        if len(self.text) < 20:
            self.setInfo(self.text + self.sender().text())
            self.getUsers()

            if self.upcase:
                self.upcase = False
                self.charsUpdate()

    def shiftClick(self):
        self.upcase = not self.upcase
        self.charsUpdate()

    def deleteClick(self):
        self.setInfo(self.text[:-1])

        if self.text:
            self.getUsers()
        else:
            self.upcase = True
            self.charsUpdate()
            self.done.hide()
            self.line1.hide()
            self.line2.hide()
            self.up.hide()
            self.down.hide()

    def lineClick(self):
        sender = self.sender()

        self.setInfo(sender.text())
        user_data = sender.property('user')
        self.selected_user['name'] = user_data['name']
        self.selected_user['account_id'] = user_data['acct']
        self.done.show()

    def upClick(self):
        if self.offset > 0:
            self.offset -= 1
            self.showLine(self.line1, 0)
            self.showLine(self.line2, 1)

    def downClick(self):
        if self.offset < (len(self.users) - 2):
            self.offset += 1
            self.showLine(self.line1, 0)
            self.showLine(self.line2, 1)

    # Protocol handlers #

    def userAnswer(self, msg):
        if not self.waiting_for_users:
            return
        answer = msg['answer']

        if answer['user'] == 'ok':
            user = msg['user']
            self.users.append({'name': user['name'], 'acct': user['acct']})
        elif answer['user'] == 'done':
            self.showUsers()
            self.waiting_for_users = False
