"""Default 800x480 sub-window with home and close buttons."""
from PyQt5.QtWidgets import QWidget, QPushButton
# , QLabel
from PyQt5 import QtCore
from PyQt5.QtCore import pyqtSignal
# from PyQt5.QtGui import QPixmap


class WindowDefaultWidget(QWidget):
    """Default 800x480 sub-window with home and close buttons.

    Signal 'go_back' and 'cancel' on closing
    """

    go_home = pyqtSignal()
    go_back = pyqtSignal()
    cancel = pyqtSignal()
    accept = pyqtSignal()

    def __init__(self, parent=None):
        """Create default 800x480 sub-window with home and close buttons.

        Args:
            parent (PyQt5.QtWidgets.QWidget): Parent widget in Qt widgets
                hierarchy. Toplevel widget if None
        Returns:
            PyQt5.QtWidgets.QWidget: Styled QWidget

        Don't forget to call 'super().__init__(parent)'.
        """
        super().__init__(parent)
        print('WindowDefaultWidget inited. Self:', self, '  parent: ', parent)

        if 'go_home' in dir(parent):
            self.go_home.connect(parent.go_home.emit)

        self.setGeometry(0, 0, 800, 480)
        self.setAttribute(QtCore.Qt.WA_StyledBackground)

        home_btn = QPushButton(None, self)
        home_btn.setFlat(True)
        home_btn.setGeometry(0, 0, 120, 72)
        home_btn.setObjectName('home')
        home_btn.clicked.connect(self.go_home.emit)
        home_btn.clicked.connect(self.cancel.emit)
        # home_btn.lower()

        close_btn = QPushButton(None, self)
        close_btn.setFlat(True)
        close_btn.setObjectName('close')
        close_btn.move(665, -45)
        # close_btn.lower()
        # close.setGeometry(725, 15, 773-725, 52-15)
        close_btn.clicked.connect(self.go_back.emit)
        close_btn.clicked.connect(self.cancel.emit)

        self.hide()
        self.go_home.connect(self.go_back.emit)
        self.go_back.connect(self.hide)

    def showEvent(self, event):  # pylint: disable-msg=C0103
        """Just extension of QT showEvent method."""
        super().showEvent(event)
        self.raise_()
