#!/bin/sh
HEAD=$(cat <<- END
# /etc/network/interfaces -- configuration file for ifup(8), ifdown(8)

# The loopback interface
auto lo
iface lo inet loopback

# Wired or wireless interfaces
auto eth0
END
)
NL='
'
DEFAULT_LUA=/home/s30/default.lua
INTERFACES=/etc/network/interfaces
MAK=""
makstring=""
need_restart=""
if [ -r $DEFAULT_LUA ] ; then
	MAK=$(sed -En 's/mac_addr\s+=\s+"([^"]+)"/\1/p' $DEFAULT_LUA)
	if [ -n "$MAK" ] ; then
		makstring="	hwaddress ether $MAK"
	fi
	DEF_IP=$(sed -En 's/ip_addr\s+=\s+"([^"]+)"/\1/p' $DEFAULT_LUA)
	DEF_MASK=$(sed -En 's/net_mask\s+=\s+"([^"]+)"/\1/p' $DEFAULT_LUA)
	DEF_GATE=$(sed -En 's/net_gateway\s+=\s+"([^"]+)"/\1/p' $DEFAULT_LUA)
	preset="$HEAD${NL}iface eth0 inet static${NL}	address $DEF_IP${NL}	netmask $DEF_MASK${NL}	gateway $DEF_GATE${NL}$makstring"
	if ! echo "$preset" | diff -q $INTERFACES.preset - ; then
		echo "$preset" > $INTERFACES.preset
		need_restart=true
	fi
fi

dhcp="$HEAD${NL}iface eth0 inet dhcp${NL}$makstring"

if ! echo "$dhcp" | diff -q $INTERFACES.dhcp - ; then
	echo "$dhcp" > $INTERFACES.dhcp
	need_restart=true
fi

fatal="$HEAD${NL}iface eth0 inet static${NL}	address 10.0.0.1${NL}	netmask 255.0.0.0${NL}	gateway 10.0.0.1${NL}$makstring"

if ! echo "$fatal" | diff -q $INTERFACES.fatal - ; then
	echo "$fatal" > $INTERFACES.fatal
	need_restart=true
fi

if [ -n "$2" ] ; then
	normal="$HEAD${NL}iface eth0 inet static${NL}	address $2${NL}	netmask $3${NL}	gateway $4${NL}$makstring"
	
	if ! echo "$normal" | diff -q $INTERFACES.normal - ; then
		echo "$normal" > $INTERFACES.normal
		need_restart=true
	fi
fi

if [ -n "$1" ] ; then
	current="$(readlink $INTERFACES)"
	newlink="$INTERFACES.$1"
	if [ -r "$newlink" ] ; then
		if [ ! "$current" = "$newlink" ] ; then
			rm -f $INTERFACES
			ln -sf "$newlink" $INTERFACES
			need_restart=true
		fi
	fi
fi

if [ -n "$need_restart" ] ; then
# shellchek recommends pgrep, but there are no pgrep in busybox
# shellcheck disable=SC2009
	if ps aux | grep -qE "root\s+\./s30" ; then
		if [ "$1" != "dhcp" ] ; then
			# udhcpc needs special signal USR2 before termination to release lease properly
			killall -USR2 udhcpc
			killall -TERM udhcpc
		fi
		ifdown -f eth0 ; sleep 1 ; ifup eth0
	fi
fi
